Package: m2r
Type: Package
Title: Interface to 'Macaulay2'
Version: 1.0.3
Authors@R: c(person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-9999-1558")),
    person("Christopher", "O'Neill", email = "musicman3320@gmail.com", role = c("aut", "cph")),
    person("Jeff", "Sommars", email = "sommars1@uic.edu", role = c("aut", "cph")))
Maintainer: David Kahle <david@kahle.io>
Description: Persistent interface to 'Macaulay2' <https://www.macaulay2.com>
    and front-end tools facilitating its use in the 'R' ecosystem. For details see 
    Kahle et. al. (2020) <doi:10.18637/jss.v093.i09>.
License: GPL-2
Depends: mpoly (>= 1.0.5)
Imports: stringr, memoise, gmp, usethis, Rcpp
SystemRequirements: Macaulay2 <https://www.macaulay2.com>
URL: https://github.com/coneill-math/m2r
BugReports: https://github.com/coneill-math/m2r/issues
Suggests: knitr, rmarkdown, microbenchmark, testthat, R.rsp
RoxygenNote: 7.3.2
LinkingTo: Rcpp
VignetteBuilder: R.rsp
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-06-10 05:19:47 UTC; david_kahle
Author: David Kahle [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9999-1558>),
  Christopher O'Neill [aut, cph],
  Jeff Sommars [aut, cph]
Repository: CRAN
Date/Publication: 2025-06-10 05:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 21:13:16 UTC; unix
Archs: m2r.so.dSYM
