# Generated by roxygen2: do not edit by hand

S3method("*",m2_ideal)
S3method("+",m2_ideal)
S3method("==",m2_ideal)
S3method("^",m2_ideal)
S3method(m2_parse_class,default)
S3method(m2_parse_class,m2_hashtable)
S3method(m2_parse_class,m2_optiontable)
S3method(m2_parse_class,m2_verticallist)
S3method(m2_parse_function,default)
S3method(m2_parse_function,m2_hashtable)
S3method(m2_parse_function,m2_ideal)
S3method(m2_parse_function,m2_image)
S3method(m2_parse_function,m2_map)
S3method(m2_parse_function,m2_monoid)
S3method(m2_parse_function,m2_monomialideal)
S3method(m2_parse_function,m2_optiontable)
S3method(m2_parse_function,m2_symbol)
S3method(m2_parse_function,m2_tocc)
S3method(m2_parse_function,m2_verticallist)
S3method(m2_parse_object_as_function,default)
S3method(m2_parse_object_as_function,m2_polynomialring)
S3method(m2_parse_object_as_function,m2_symbol)
S3method(print,m2_array)
S3method(print,m2_boolean)
S3method(print,m2_complex)
S3method(print,m2_float)
S3method(print,m2_hashtable)
S3method(print,m2_ideal)
S3method(print,m2_ideal_list)
S3method(print,m2_image)
S3method(print,m2_integer)
S3method(print,m2_list)
S3method(print,m2_matrix)
S3method(print,m2_module)
S3method(print,m2_option)
S3method(print,m2_pointer)
S3method(print,m2_polynomialring)
S3method(print,m2_sequence)
S3method(print,m2_string)
S3method(print,m2_symbol)
S3method(str_m2,default)
S3method(str_m2,m2_polynomialring)
export("m2_meta<-")
export("m2_name<-")
export(LLL)
export(LLL.)
export(dimension)
export(edit_r_environ)
export(enter_m2)
export(factor_n)
export(factor_n.)
export(factor_poly)
export(factor_poly.)
export(gb)
export(gb.)
export(gb_)
export(gb_.)
export(get_m2_con)
export(get_m2_connection)
export(get_m2_gmp)
export(get_m2_path)
export(get_m2_port)
export(get_m2_procid)
export(has_m2_connection)
export(ideal)
export(ideal.)
export(ideal_)
export(ideal_.)
export(is.m2)
export(is.m2_array)
export(is.m2_grobner_basis)
export(is.m2_ideal)
export(is.m2_ideal_list)
export(is.m2_ideal_list_pointer)
export(is.m2_ideal_pointer)
export(is.m2_list)
export(is.m2_matrix)
export(is.m2_matrix_pointer)
export(is.m2_module)
export(is.m2_option)
export(is.m2_pointer)
export(is.m2_polynomialring)
export(is.m2_polynomialring_pointer)
export(is.m2_sequence)
export(is.ring)
export(m2)
export(m2.)
export(m2_coefrings)
export(m2_exists)
export(m2_getwd)
export(m2_kernel)
export(m2_kernel.)
export(m2_length)
export(m2_ls)
export(m2_matrix)
export(m2_matrix.)
export(m2_meta)
export(m2_name)
export(m2_numcols)
export(m2_numrows)
export(m2_parse)
export(m2_rm)
export(m2_structure)
export(m2_termorders)
export(m2_toggle_gmp)
export(m2r_cloud_url)
export(m2r_version_number)
export(mixed_volume)
export(primary_decomposition)
export(primary_decomposition.)
export(quotient)
export(quotient.)
export(radical)
export(radical.)
export(reset_m2)
export(ring)
export(ring.)
export(ring_)
export(ring_.)
export(saturate)
export(saturate.)
export(set_m2_path)
export(snf)
export(snf.)
export(solve_system)
export(solve_system.)
export(start_m2)
export(stop_m2)
export(str_m2)
export(use_ring)
import(gmp)
import(mpoly)
importFrom(Rcpp,sourceCpp)
importFrom(memoise,forget)
importFrom(memoise,memoise)
importFrom(stringr,fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(usethis,edit_r_environ)
useDynLib(m2r)
