# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

m2_tokenize_cpp <- function(s) {
    .Call('_m2r_m2_tokenize_cpp', PACKAGE = 'm2r', s)
}

